(function ( $ ) {
	"use strict";

	$(function () {

		var Chart = function() {

            var chart = this;

            this.options = {
                chartType : $("#weblator_charts_chart_type")
            }

            function init(){

                showLegendOption();
                showLegendOptionsSecondary();
                showPercentageOptions();

                //Load Style Options
                if ($(".new-chart").length)
                loadStyleSettings(chart.options.chartType.val());

                chart.options.chartType.change(function(){
                    loadStyleSettings(chart.options.chartType.val());
                    showLegendOption();
                    showPercentageOptions();
                });

                $("button.add-option").click(function(){
                    addNewOption();
                });

                $("table.chart-options tbody").sortable({
                    handle: ".drag",
                    placeholder: "ui-state-highlight",
                    update: reorderOptions,
                    items: "tr"
                });

                $(".save-button").click(function(){
                    saveChart(this);
                });

                $("body").on("change", "input[type='checkbox']", function(){
                    $(this).val((Number(this.checked)));
                });

                $(".delete_chart").click(function(e){
                    e.preventDefault();
                   deleteChart(this);
                });

                $("table.chart-options").on("click", ".poll-option-remove", function(){
                    deletePollOption(this);
                });

                $("#chart_legend").change(function(){
                   showLegendOptionsSecondary();
                });

                if ($(".new-chart").length){
                reorderOptions();
                showPickers();
                }
            }

            function showPercentageOptions(){

                if (chart.options.chartType.val() == 7){
                    $(".percentage-options").css("display", "table-row");
                }else {
                    $(".percentage-options").css("display", "none");
                }

            }

            function deletePollOption(e){

                if ($(".poll-option-remove:visible").length > 1)
                    $(e).parent().parent().remove();

                reorderOptions();
            }

            function showLegendOption(){

                if (chart.options.chartType.val() == 2 || chart.options.chartType.val() == 3 || chart.options.chartType.val() == 6){
                    $(".legend-option").css("display", "table-row");
                }else {
                    $(".legend-option").css("display", "none");

                }

                showLegendOptionsSecondary();
            }

            function showLegendOptionsSecondary(){


                if ($(".legend-option:visible").length){


                    if ($("#chart_legend").is(":checked")){
                        $(".legend-option-secondary").css("display", "table-row");

                    }else{
                        $(".legend-option-secondary").css("display", "none");

                    }


                }else {

                    $(".legend-option-secondary").css("display", "none");

                }
            }

            function deleteChart(e) {

                var id = $(e).attr("data-chart-id");

                var data = {
                    action: "delete_chart",
                    chart_id : id
                };

                if (confirm('Are you sure you want to delete this chart?')) {
                    $.post(ajaxurl, data, function(response){
                        window.location = "?page=charts";
                    });
                }

            }

            function showColorSettings(chartTypeID){

                switch (parseInt(chartTypeID)) {

                    case 6:
                    case 2:
                    case 3:
                    case 7:
                        $("td.colour-option").css("display", "table-cell");
                        break;

                    default:
                        $("td.colour-option").css("display", "none");


                }

            }

            function loadStyleSettings(chartTypeID){

                showColorSettings(chartTypeID);

                $.post(ajaxurl, {action:"loadStyleSettings", chartTypeID:chartTypeID, chartID:$("input[name='save']").attr("data-chart-id")}, function(data){

                    var json = $.parseJSON(data);

                    for(var i = 0; i < json.length; i++){

                        var template = Handlebars.compile($("#optionRow").html());
                        var html = html + template(json[i]);
                    }


                    $(".chart-settings tbody").html(html);

                    showPickers();

                });
            }

            function addNewOption(){

                var template = Handlebars.compile($("#optionTmpl").html());
                var html = template();

                $("table.chart-options tbody").append(html);
                showColorSettings(chart.options.chartType.val());
                destroyPickers();
                showPickers();
                reorderOptions();
            }

            function showPickers(){

                $(".fill input").spectrum({
                    showAlpha:true,
                    preferredFormat: "rgb",
                    showInput: true
                });

                $(".fill input").show();

            }

            function destroyPickers(){

                $(".fill input").spectrum("destroy");
            }

            function reorderOptions () {

                $("table.chart-options tr").each(function(i){

                    $(this).attr("data-order", i+1);

                });

                destroyPickers();
                showPickers();
            }

            function saveChart(e){

                var action = ($(e).hasClass("edit") ? "update_chart" : "save_chart");

                //Disable Update Button
                $(e).attr("disabled", "disabled");

                //If a message is visible then slide it up
                if ($("#message").is(':visible')){
                    $("#message").slideUp();
                }

                //Show the animated Spinner
                $(".delete-action").css("visibility", "visible");
                $(".delete-action span.spinner").css("display", "inline-block");

                //Get Poll Options
                var options = [];
                $("table.chart-options tr").each(function(){
                    var self = this;
                    options.push([
                        parseInt($(self).attr("data-order")),
                        $(self).find("input[name='chart_option']").val(),
                        $(self).find("input[name='chart_option']").attr("data-id"),
                        $(self).find("input[name='chart_colour']").val(),
                        $(self).find("input[name='chart_value']").val()
                    ]);
                });

                for (var i=0; i < options.length; i++){

                    if (isNaN(parseInt(options[i][4]))){

                        //There was an error adding the chart so show error message
                        $("#message").removeClass("updated").addClass("error")
                            .html("<p>Option Values must be an Integer</p>");

                        $("#message").slideDown();

                        //Remove disabled attribute from update button
                        $(e).removeAttr("disabled");

                        $(".delete-action span.spinner").css("display", "none");
                        return;
                    }

                    if (chart.options.chartType.val() == 7 && parseInt(options[i][4]) > 100 && $("[name='chart_percentage_values']").val() == 1){

                        $("#message").removeClass("updated").addClass("error")
                            .html("<p>Option Values must be 100 or below</p>");

                        $("#message").slideDown();

                        //Remove disabled attribute from update button
                        $(e).removeAttr("disabled");

                        $(".delete-action span.spinner").css("display", "none");
                        return;
                    }
                }



                //Make sure all input that is necessary is complete
                if (!validateInput(options)){
                    $(e).removeAttr("disabled");
                    $(".delete-action span.spinner").css("display", "none");
                    return;
                }


                var styleValues = getStyleValues();
                var data = getPollData(e, action, options, styleValues);

                //Post data to server
                $.post(ajaxurl, data, function(r){

                    if (r == 0){

                        //There was an error adding the chart so show error message
                        $("#message").removeClass("updated").addClass("error")
                            .html("<p>There was an error adding your chart please try again</p>");

                        $("#message").slideDown();

                        //Remove disabled attribute from update button
                        $(e).removeAttr("disabled");

                        $(".delete-action span.spinner").css("display", "none");

                    } else {

                        if (action == "save_chart")
                            window.location = "?page=charts&action=edit&m=1&edit_chart="+r;
                        else
                            window.location = "?page=charts&action=edit&m=1&edit_chart="+$(e).attr("data-chart-id");

                    }

                });
            }

            function validateInput(options){

                if ($("[name='weblator_chart_name']").val() == ""){

                    $("#message").removeClass("updated").addClass("error")
                        .html("<p>Please give your chart a title</p>").slideDown();

                    $(".delete-action i").css("visibility", "hidden");
                    $(self).removeAttr("disabled");
                    return false;
                }

                if (options.length == 1 && options[0][1] == ""){

                    $("#message").removeClass("updated").addClass("error")
                        .html("<p>Please add some options</p>");

                    $(".delete-action i").css("visibility", "hidden");
                    $(self).removeAttr("disabled");

                    $("#message").slideDown();
                    return false;
                }

                for(var i = 0; i < options.length; i++){

                    if (options[i][1] == ""){
                        $("#message").removeClass("updated").addClass("error")
                            .html("<p>Please do not leave any options blank</p>");

                        $(".delete-action i").css("visibility", "hidden");
                        $(self).removeAttr("disabled");

                        $("#message").slideDown();
                        return false;

                    }

                }


                return true;

            }

            function getStyleValues(){

                var values = [];
                $(".chart-settings input, .chart-settings select").each(function(){

                    var obj = {
                        key: $(this).attr("name"),
                        value: $(this).val(),
                        id: $(this).attr("data-style-id")
                    };

                    values.push(obj);

                });

                return values;
            }

            function getPollData(e, action, options, style){

                return {

                    chart_id : $(e).attr("data-chart-id"),
                    action : action,
                    options : options,
                    name : $("[name='weblator_chart_name']").val(),
                    chart : $("[name='weblator_charts_chart_type']").val(),
                    is_live : $('input[name="poll_status"]:checked').val(),
                    legend : $("[name='chart_legend']").val(),
                    legend_position : $("[name='weblator_charts_legend_position']").val(),
                    legend_font_size : $("[name='weblator_charts_legend_font_size']").val(),
                    legend_font_style : $("[name='weblator_charts_chart_legend_font_style']").val(),
                    legend_font_colour : $("[name='chart_legend_font_colour']").val(),
                    chart_percentage_values : $("[name='chart_percentage_values']").val(),
                    maxWidth : $('input[name="chart_max_width"]').val(),
                    styles : style
                }
            }

            init();
        }, c = new Chart();

	});

}(jQuery));